package Mat;

import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.greenstone.gsdl3.core.MessageRouter;
import org.greenstone.gsdl3.util.DBInfo;
import org.greenstone.gsdl3.util.GDBMWrapper;
import org.greenstone.gsdl3.util.GlobalProperties;
import org.greenstone.gsdl3.util.GSFile;
import org.greenstone.gsdl3.util.GSXML;
import org.greenstone.gsdl3.util.XMLConverter;



public class DescribeMessager {
	protected String site_name = "localsite";
	private GDBMWrapper gdbm_src = new GDBMWrapper();
	/** message router object to pass requests messages to and which 
	 * will process them.*/
	protected MessageRouter mr = null;
	
	/** container Document to create XML Nodes */
	protected Document doc=null;
	/** a converter class to parse XML and create Docs */
	protected XMLConverter converter=null;
	
	protected final static String mrSubsetOptions = //messageRouter 
		"collectionList serviceClusterList serviceList siteList"; 
	protected final static String csSubsetOptions = //collections and serviceClusters 
		"metadataList serviceList displayItemList";
	protected final static String serviceSubsetOptions = //services
		"paramList displayItemList";
	final int MESSAGEROUTER = 1, COLLECTION = 2, 
		SERVICECLUSTER = 3, SERVICE = 4;
	
	
	public DescribeMessager() {
	    String gsdl3_home = GlobalProperties.getGSDL3Home();
		if (gsdl3_home == null || gsdl3_home.equals("")) {
		    System.err.println("Couldn't access GSDL3Home from GlobalProperties.getGSDL3HOME, can't initialize the SOAP Server.");
		    return;
		}
		
		String site_home = GSFile.siteHome(gsdl3_home, this.site_name);
		
		File site_file = new File(site_home);
		if (!site_file.isDirectory()) {
		    System.err.println("The site directory "+site_file.getPath()+" doesn't exist. Can't initialize the SOAP Server.");
		    return;
		} 
		this.converter = new XMLConverter();
		this.doc = this.converter.newDOM();
		
		mr = new MessageRouter();
		mr.setSiteName(this.site_name);
		mr.configure();
    }
	
	//Describe messages sent to MESSAGEROUTER
	//By default: sends a describe request message to the MessageRouter, with
	//lang=en (and all MessageRouter subsetOptions set).
	public String describe() 
	{
		//describe(to=MR, lang=en, no subsetOptions, from valid mrSubsetOptions)
		return describe("", "", "", mrSubsetOptions);
	}
	
	public String describe(String subsetOption) 
	{
		return describe("", "", subsetOption, mrSubsetOptions);
	}
	
	public String describe(String lang, String subsetOption) 
	{
		return describe("", lang, subsetOption, mrSubsetOptions);
	}
	
	
	//Describe messages sent to COLLECTIONS and SERVICERACKS
	public String describeServiceRack(String toSC, String lang, 
			String subsetOption)
	{
		return describe(toSC, lang, subsetOption, csSubsetOptions);
	}
	
	
	public String describeCollection(String toColl, String lang, 
			String subsetOption)
	{
		return describe(toColl, lang, subsetOption, csSubsetOptions);
	}
	
	//Describe Messages sent to Services (or Collection/Services) 
	public String describeCollService(String toColl, String toService, 
			String lang, String subsetOption) {
		/*String message =  
			"<message><request lang=\"en\" to=\"gs2mgppdemo/FieldQuery\" type=\"describe\" /></message>";
				
			String response = mr.process(message);
			System.out.println("Response to: " + message + ":\n");
			System.out.println(response);
		*/	
		return describe(toColl + "/" + toService, 
				lang, subsetOption, serviceSubsetOptions);
	}
	
	public String describeService(
			String toService, String lang, String subsetOption) 
	{
		return describe(toService, lang, subsetOption, serviceSubsetOptions);
	}
	
	//GET POSSIBLE SUBSET OPTIONS FOR THE VARIOUS RECEIVERS (MessageRouter,
	//Services, ServiceRack/Collections
	
	//Subset options that can be specified when asking the messageRouter to
	//describe itself
	public String getMessageRouterSubsetOptions() {
		return mrSubsetOptions;
	}
	
	//Possible subset options for ServiceClusters and Collections
	public String getCSSubsetOptions() {
		return csSubsetOptions;
	}
	
	//Possible subset options for services
	public String getServiceSubsetOptions() {
		return serviceSubsetOptions;
	}
	
	//If public, this method would give full access: a describe message that 
	//lets the user specify all the details of who the receiver is, and what 
	//details are requested.
	
	/*
	protected String describe(String to, String lang, 
			String subsetOption, String validSubsetOptions)
	{
		//Create message element: <message></message>
		Element message = this.doc.createElement(GSXML.MESSAGE_ELEM);
		//<message><request lang="en" to="" type="describe" uid="" /></message>
		Element request = GSXML.createBasicRequest(
				this.doc, GSXML.REQUEST_TYPE_DESCRIBE, to, lang, "");
		
		//Check if only a subset of this Module Interface's data is asked
		//to be described
		if(subsetOption != "") {
			//Now deal with the value for subset param:
			//only deal with valid params for subset of to-ModuleInterface
			if(validSubsetOptions.indexOf(subsetOption) == -1)
				return "Invalid List to be described. Choose one of:\n"
					+ validSubsetOptions;
			
			//else, append <paramList />
			Element param_list = this.doc.createElement(
					GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
			//append <param name="subset" value=paramValue />
			//createParam(Document, name, value); 
			//Name needs to be "subset", but this can be either GSXML.SUBSET_PARAM
			//or GSXML.SYSTEM_SUBSET_ATT. It's the first one probably.
			param_list.appendChild(GSXML.createParameter(
							this.doc, GSXML.SUBSET_PARAM, subsetOption));
			request.appendChild(param_list);
		}
		message.appendChild(request);
		
		//System.out.println(this.converter.getPrettyString(message));
		
		//Let the messagerouter process the request message and get the response
		Element response = mr.process(message.getFirstChild());
		//String response = mr.process(this.converter.getPrettyString(message));
		//this.converter.getPrettyString(arg0);
		//mr.p
		//Return it as a String formatted for display
		System.out.println(this.converter.getPrettyString(response));
		
		return null;
		//return this.converter.getPrettyString(response);
		//return this.converter.getString(response);
		//return mr.process(this.converter.getString(message));
	}
	*/
	
	
	
	protected String describe(String to, String lang, 
			String subsetOption, String validSubsetOptions)
	{
		//Create message element: <message></message>
		Node message = this.doc.createElement(GSXML.MESSAGE_ELEM);
		//<message><request lang="en" to="" type="describe" uid="" /></message>
		Node request = GSXML.createBasicRequest(
				this.doc, GSXML.REQUEST_TYPE_DESCRIBE, to, lang, "");
		
		//Check if only a subset of this Module Interface's data is asked
		//to be described
		if(subsetOption != "") {
			//Now deal with the value for subset param:
			//only deal with valid params for subset of to-ModuleInterface
			if(validSubsetOptions.indexOf(subsetOption) == -1)
				return "Invalid List to be described. Choose one of:\n"
					+ validSubsetOptions;
			
			//else, append <paramList />
			Node param_list = this.doc.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
			//append <param name="subset" value=paramValue />
			//createParam(Document, name, value); 
			//Name needs to be "subset", but this can be either GSXML.SUBSET_PARAM
			//or GSXML.SYSTEM_SUBSET_ATT. It's the first one probably.
			param_list.appendChild(GSXML.createParameter(this.doc, GSXML.SUBSET_PARAM, subsetOption));
			request.appendChild(param_list);
		}
		message.appendChild(request);
		
		//System.out.println(this.converter.getPrettyString(message));
		
		//Let the messagerouter process the request message and get the response
		Node response = mr.process(message);
		//String response = mr.process(this.converter.getPrettyString(message));
		//this.converter.getPrettyString(arg0);
		//mr.p
		//Return it as a String formatted for display
		Element res = (Element) response;
		System.out.println(this.converter.getPrettyString(res));
		System.out.println("break1");
		
    	String gsdl3_home = GlobalProperties.getGSDL3Home();
    	if (gsdl3_home == null || gsdl3_home.equals("")) {
    	    System.err.println("Couldn't access GSDL3Home from GlobalProperties.getGSDL3HOME, can't initialize the SOAP Server.");
    	}
    	
    	String site_home = GSFile.siteHome(gsdl3_home, this.site_name);
		String gdbm = GSFile.collectionDatabaseFile(site_home,"sampleco","sampleco","gdbm");

		/*
		try{
		System.out.println("break2.1");
		//System.out.println(this.gdbm_src.openDatabase("gs2mgppdemo",GDBMWrapper.READ));
		}catch (Exception ex) {ex.printStackTrace();}
		*/
		//System.out.println(gdbm);
		System.out.println("break2.2");
		if (!this.gdbm_src.openDatabase(gdbm,GDBMWrapper.READ)) {
		//if(false){
			System.out.println("break31");
		    System.out.println("Could not open GDBM database!");
		    
		}
		
		else{
			System.out.println("break32");

			
		    String info = this.gdbm_src.getValue("browselist");
		
		    if (info == null) {
		    	System.out.println("cannot locate the list");
		    }
		    else{
			
			if (info == null) {
				System.out.println("the db does not contain any info");
			}
		    
			
			ArrayList children = new ArrayList();
			StringTokenizer st = new StringTokenizer(info, ";");
			//StringTokenizer st = new StringTokenizer(st2, ";");
			//System.out.println(info);

			while (st.hasMoreTokens()) {
				String part = st.nextToken(";");
				//System.out.println("..."+part+".....1");
				
				
				if(part.contains("<contains>")){
					part = part.replace("<contains>", "");
				}
				else if(part.contains("<thistype>")){
					//System.out.println("here");
					int location = part.indexOf("<thistype>");
					part = part.substring(0,location-1);
				}
				System.out.println("..."+part+".....2");
			    //String child_id = st.nextToken().replaceAll("\"", node_id);
			    //children.add(child_id);
			}
			
			gdbm_src.closeDatabase();
			System.out.println("start..................");
			//System.out.println(info);
			System.out.println("end..................");
			//parseMetadata(children,"all");
			//TotalDoc = children.size();
			//System.out.println("Total Doc:"+TotalDoc);
			//status = true;
			//Display();
		    //}
		    }
		}
		
		return "";
		//return this.converter.getPrettyString(response);
		//return this.converter.getString(response);
		//return mr.process(this.converter.getString(message));
	}
	//UNUSED:
	public static String describeExampleResponse() {
		XMLConverter converter = new XMLConverter();
		Document doc = converter.getDOM("SampleServiceResponse.xml");
		return converter.getPrettyString(doc);
	}

	
	
	public static void main(String[] args) {
		DescribeMessager dm = new DescribeMessager();
		String response[] = new String[3];
		
		response[0] = dm.describe();
		//response[1] = dm.describeMessageRouter("", "serviceList"); //""->lang=en
		response[1] = dm.describeService("TextQuery", "en", "");
		response[2] = dm.describeCollection("gs2mgdemo", "en", "metadataList");
		//response[3] = dm.describeCollection("gberg", "en", "serviceList");
		//response[4] = dm.describeService("DocumentMetadataRetrieve", "", "");
		//response[3] = dm.describeService("DocumentContentRetrieve", "en", "");

		//French doesn't seem to have any effect
		//response[3] = dm.sendDescribe("", "fr"); 
		/*
		for(int i = 0; i < response.length; i++)
			System.out.println(
					(response[i]==null) ? "no response" : response[i]);
		System.exit(0);
		*/
	}
}